import unittest

from loguru import logger
from sybil_engine.config.app_config import set_network

from web3_wizzard_lib.core.modules.stargate.farming import StargateFarmingModule
from web3_wizzard_lib.core.modules.stargate.pool import get_pool_balance_for_chain
from tests.core.modules.__init__ import test_account, init_set_test_config, test_min_native_balance

# Module [STARGATE_FARMING, 14] Stargate farming
test_stargate_farming_config = {
    'chain': 'BASE',
    'bridge_amount_interval': {'from': 1, 'to': 1},
    'all_balance_mode': 0,
    # 0 - all balance ignores CHAIN stakes blockchain with maximum balance
    # 1 - all balance stakes all balance from CHAIN
    'operation': {'ADD_TO_POOL', 'DEPOSIT', 'WITHDRAW', 'REDEEM_FROM_POOL'},
    # 'operation': {'REDEEM_FROM_POOL'},
    'token': 'ETH',  # ['ETH', 'USDC']
    # 'operation': {'WITHDRAW', 'REDEEM_FROM_POOL'}
    'operation_sleep_interval': {'from': 0, 'to': 0}
}


class TestStargateFarming(unittest.TestCase):
    init_set_test_config()

    def test_shouldStargateFarming(self):
        logger.info("Test module [STARGATE_FARMING, 14]")

        test_stargate_farming_config['bridge_amount_interval'] = {'from': 1, 'to': 1}
        stargate_farming = StargateFarmingModule(test_min_native_balance, test_account)

        try:
            stargate_farming.execute(*stargate_farming.parse_params(test_stargate_farming_config), test_account)
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")

    def test_shouldStargateFarmingWithAllBalance(self):
        logger.info("Test module [STARGATE_FARMING, 14]")

        test_stargate_farming_config['bridge_amount_interval'] = 'all_balance'
        stargate_farming = StargateFarmingModule(test_min_native_balance, test_account)

        try:
            stargate_farming.execute(*stargate_farming.parse_params(test_stargate_farming_config), test_account)
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")

    def test_shouldStargateFarmingWithAllBalanceMode1(self):
        logger.info("Test module [STARGATE_FARMING, 14]")

        test_stargate_farming_config['bridge_amount_interval'] = 'all_balance'
        test_stargate_farming_config['all_balance_mode'] = 1

        stargate_farming = StargateFarmingModule(test_min_native_balance, test_account)

        try:
            stargate_farming.execute(*stargate_farming.parse_params(test_stargate_farming_config), test_account)
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")

    def test_shouldStargateFarmingStepByStep(self):
        set_network('LOCAL')

        logger.info("Test module [STARGATE_FARMING, 14]")

        test_stargate_farming_config['bridge_amount_interval'] = {'from': 10, 'to': 10}
        test_stargate_farming_config['all_balance_mode'] = 1

        stargate_farming = StargateFarmingModule(test_min_native_balance, test_account)

        try:
            #PREPARE
            test_stargate_farming_config['operation'] = {'REDEEM_FROM_POOL'}
            stargate_farming.execute(*stargate_farming.parse_params(test_stargate_farming_config), test_account)

            chain, pool_balance_1, web3 = get_pool_balance_for_chain('BASE', 'ETH', test_account)

            test_stargate_farming_config['operation'] = {'ADD_TO_POOL'}
            stargate_farming.execute(*stargate_farming.parse_params(test_stargate_farming_config), test_account)

            chain, pool_balance_2, web3 = get_pool_balance_for_chain('BASE', 'ETH', test_account)

            self.assertAlmostEqual(
                float(pool_balance_2.minus(pool_balance_1).readable()),
                10.0,
                delta=0.02
            )

            test_stargate_farming_config['operation'] = {'DEPOSIT'}
            stargate_farming.execute(*stargate_farming.parse_params(test_stargate_farming_config), test_account)

            test_stargate_farming_config['operation'] = {'WITHDRAW'}
            stargate_farming.execute(*stargate_farming.parse_params(test_stargate_farming_config), test_account)

            test_stargate_farming_config['operation'] = {'REDEEM_FROM_POOL'}
            stargate_farming.execute(*stargate_farming.parse_params(test_stargate_farming_config), test_account)

            chain, pool_balance_result, web3 = get_pool_balance_for_chain('BASE', 'ETH', test_account)

            self.assertAlmostEqual(
                first=pool_balance_result.readable(),
                second=pool_balance_1.readable(),
                delta=0.01
            )
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")
